#include "mplayer.h"
#include <QProcess>
#include <QColor>
#include <QPalette>
#include <QTimer>
#include <QMenu>
#include <QCursor>
#include <iostream>
#include <QMouseEvent>
using namespace std;

MPlayer::MPlayer(QWidget *parent) :
    QWidget(parent)
{
    w = h = 1;

    //Create timer
    timer = new QTimer(this);
    vol_timer = new QTimer(this);
    vol_timer->setSingleShot(true);

    //Create layer
    layer = new QWidget(this);

    //Set state
    state = STOPPING;
    is_waiting = false;

    //Set color
    setPalette(QPalette(QColor(0, 0, 0)));
    setAutoFillBackground(true);

    //Create mplayer process
    process = new QProcess(this);
    process->setProcessChannelMode(QProcess::MergedChannels);
    connect(process, SIGNAL(finished(int)), this, SLOT(onFinished(int)));
    connect(process, SIGNAL(readyReadStandardOutput()), this, SLOT(readOutput()));
    connect(timer, SIGNAL(timeout()), this, SLOT(updateTime()));

    //set menu
    QMenu* ratio_menu = new QMenu(tr("Ratio"));
    ratio_menu->addAction("4:3", this, SLOT(setRatio_4_3()));
    ratio_menu->addAction("16:9", this, SLOT(setRatio_16_9()));
    ratio_menu->addAction("16:10", this, SLOT(setRatio_16_10()));
    ratio_menu->addAction(tr("Default"), this, SLOT(setRatio_0()));
    menu = new QMenu(this);
    menu->addMenu(ratio_menu);
    setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this, SIGNAL(customContextMenuRequested(const QPoint&)),
            this, SLOT(showMenu(const QPoint&)));
}

//resize layer when window size changes
void MPlayer::resizeEvent(QResizeEvent *)
{
    if (state != STOPPING)
        resizeLayer();
}

void MPlayer::resizeLayer()
{
    int pos_x, pos_y, size_width, size_height;
    int win_width = width();
    int win_height = height();

    size_width = win_width;
    size_height = size_width * h / w;
    if (size_height > win_height)
    {
        size_height = win_height;
        size_width = size_height * w / h;
    }

    pos_x = (win_width - size_width) / 2;
    pos_y = (win_height - size_height) / 2;

    layer->move(pos_x, pos_y);
    layer->resize(size_width, size_height);
}

/*Set mplayer to fullscreen*/
void MPlayer::mouseDoubleClickEvent(QMouseEvent* e)
{
    if (e->buttons() == Qt::LeftButton)
        emit fullScreen();
}

/* Open a new video file. MPlayer::openFile sends a message to get length
 * and the result will be read in MPlayer::cb_start.
 */
void MPlayer::openFile(QString& filename)
{
    //start MPlayer if stopping
    if (state != STOPPING)
    {
        is_waiting = true;
        wait_to_play = filename;
        stop();
        return;
    }

    //Set time to 0 and cache state
    emit timeChanged(0);
    emit catching(true);

    QStringList args;
    args << "-quiet";
    args << "-slave";
    args << "-vo" << "xv";
    args << "-ao" << "alsa";
    args << "-wid" << QString::number(layer->winId());
    args << filename;
    process->start("mplayer", args);

    //set state
    state = VIDEO_PLAYING;
    is_caching = true;
    process->write("get_time_length\n");
}

void MPlayer::cb_start(QString& msg)
{
    msg.chop(4);
    length = msg.mid(11).toInt();
    progress = 0;
    timer->start(1000);
    emit lengthChanged(length);
    emit played();
    emit catching(false);
}


/* Change state between pausing and playing. */
void MPlayer::changeState()
{
    switch (state)
    {
    case VIDEO_PLAYING:
        timer->stop();
        process->write("pause\n");
        state = VIDEO_PAUSING;
        emit paused();
        break;
    case VIDEO_PAUSING:
        timer->start(1000);
        process->write("pause\n");
        state = VIDEO_PLAYING;
        emit played();
    default:break;
    }
}


/* MPlayer::stop sends a message to quit mplayer. When the mplayer
 * receive the message or the video is at the end, it will automatically
 * exit and emit Finished() signal. Then something will be done
 * in MPlayer::onFinished.
 */
void MPlayer::stop()
{
    if (state != STOPPING)
        process->write("q\n");
    process->waitForFinished();
}

void MPlayer::onFinished(int)
{
    state = STOPPING;
    timer->stop();

    if (is_waiting)
    {
        is_waiting = false;
        openFile(wait_to_play);
    }
    else
        emit stopped();
}


void MPlayer::setVolume(int volume)
{
    char msg[16];
    if (state == STOPPING)
        return;

    //not set volume too frequency or mplayer may die
    if (vol_timer->isActive())
        return;
    vol_timer->start(100);

    qsnprintf(msg, 16, "volume %d 1\n", volume);
    process->write(msg);
}

void MPlayer::readOutput()
{
    while (process->canReadLine())
    {
        QString message = process->readLine();

        if (message.startsWith("VO:"))
            cb_ratioChanged(message);

        else if (message.startsWith("ANS_LENGTH="))
            cb_start(message);

        else if (message.startsWith("ANS_TIME_POSITION="))
            cb_updateTime(message);

        else if (message.startsWith("Cache not filling"))
        {
            if (!is_caching)
            {
                is_caching = true;
                emit catching(true);
            }
        }
    }
}

/*Update time every 1s. MPlayer::updateTime() will send a
 * message to get position and the result will be read in
 * MPlayer::cb_changeState.
 */
void MPlayer::updateTime()
{
    if (is_caching)
    {
        is_caching = false;
        emit catching(false);
    }
    process->write("get_time_pos\n");
}


void MPlayer::cb_updateTime(QString& msg)
{
    msg.chop(3);
    progress = msg.mid(18).toInt();
    if (progress % UPDATE_FREQUENCY == 0)
        emit timeChanged(progress);
}

/*set progress*/
void MPlayer::setProgress(int pos)
{
    //Ignore if stopping
    if (state == STOPPING)
        return;

    //Ignore if the time is equal to the progress
    if (pos == progress)
        return;

    //Set progress
    progress = pos;
    char msg[16];
    qsnprintf(msg, 16, "seek %d 2\n", pos);
    process->write(msg);
}

//set video's ratio
void MPlayer::setRatio_16_9()
{
    static QByteArray msg = "switch_ratio " + QByteArray::number(16.0 / 9.0) + '\n';
    if (state == STOPPING)
        return;
    process->write(msg);
}

void MPlayer::setRatio_16_10()
{
    if (state == STOPPING)
        return;
    process->write("switch_ratio 1.6\n");
}

void MPlayer::setRatio_4_3()
{
    static QByteArray msg = "switch_ratio " + QByteArray::number(4.0 / 3.0) + '\n';
    if (state == STOPPING)
        return;
    process->write(msg);
}

void MPlayer::setRatio_0()
{
    if (state == STOPPING)
        return;
    process->write("switch_ratio 0\n");
}

void MPlayer::cb_ratioChanged(QString& msg)
{
    QStringList size = msg.section(' ', 4, 4).split('x');
    w = size.takeFirst().toInt();
    h = size.takeFirst().toInt();
    resizeLayer();
}

//Show right-button menu
void MPlayer::showMenu(const QPoint&)
{
    menu->exec(QCursor::pos());
}
