#include "player.h"
#include "ui_player.h"
#include "string.h"
#include "stdio.h"
#include "mplayer.h"
#include "playlist.h"
#include "webvideo.h"
#include <QDir>
#include <QTimer>
#include <QCloseEvent>
#include <QMouseEvent>
#include <QApplication>
#include <QDesktopWidget>

Player::Player(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::Player)
{
    ui->setupUi(this);

    //Add player layout
    playerLayout = new QHBoxLayout;
    ui->stackedWidget->widget(0)->setLayout(playerLayout);

    //Add MPlayer frame
    mplayer = new MPlayer;
    playerLayout->addWidget(mplayer, 1);
    mplayer->installEventFilter(this);
    mplayer->setMouseTracking(true);
    mplayer->getLayer()->installEventFilter(this);
    mplayer->getLayer()->setMouseTracking(true);

    //Add Playlist
    playlist = new Playlist;
    playerLayout->addWidget(playlist);

    //Add WebVideo
    webvideo = new WebVideo;
    ui->stackedWidget->addWidget(webvideo);

    //Connect
    connect(ui->playButton, SIGNAL(clicked()), mplayer, SLOT(changeState()));
    connect(ui->stopButton, SIGNAL(clicked()), this, SLOT(onStopButton()));
    connect(ui->progressBar, SIGNAL(valueChanged(int)), this, SLOT(onPBarChanged(int)));
    connect(ui->volumeSlider, SIGNAL(valueChanged(int)), mplayer, SLOT(setVolume(int)));
    connect(ui->hideButton, SIGNAL(clicked()), this, SLOT(hidePlaylist()));
    connect(ui->tabButton, SIGNAL(clicked()), this, SLOT(changeTabToPlayer()));
    connect(ui->webTabButton, SIGNAL(clicked()), this, SLOT(changeTabToWeb()));
    connect(mplayer, SIGNAL(played()), this, SLOT(setIconToPause()));
    connect(mplayer, SIGNAL(paused()), this, SLOT(setIconToPlay()));
    connect(mplayer, SIGNAL(stopped()), this, SLOT(onStopped()));
    connect(mplayer, SIGNAL(timeChanged(int)), this, SLOT(onProgressChanged(int)));
    connect(mplayer, SIGNAL(lengthChanged(int)), this, SLOT(onLengthChanged(int)));
    connect(mplayer, SIGNAL(fullScreen()), this, SLOT(setFullScreen()));
    connect(mplayer, SIGNAL(catching(bool)), this, SLOT(onCatching(bool)));
    connect(playlist, SIGNAL(fileSelected(QString&)), mplayer, SLOT(openFile(QString&)));
    connect(playlist, SIGNAL(needPause(bool)), this, SLOT(onNeedPause(bool)));
    connect(webvideo, SIGNAL(newUrl(QString&,QString&)), playlist, SLOT(addFile(QString&,QString&)));
    connect(webvideo, SIGNAL(newUrlAndPlay(QString&,QString&)), playlist, SLOT(addFileAndPlay(QString&,QString&)));

    //Set skin
    setSkin("default");

    no_play_next = false;
    is_fullscreen = false;
}

Player::~Player()
{
    delete ui;
}

void Player::closeEvent(QCloseEvent* e)
{
    mplayer->stop();
    e->accept();
}

//Hide playlist
void Player::hidePlaylist()
{
    if (playlist->isHidden())
    {
        playlist->show();
        ui->hideButton->setText(">>");
    }
    else
    {
        playlist->hide();
        ui->hideButton->setText("<<");
    }
}

void Player::setFullScreen()
{
    if (is_fullscreen)
    {
        showNormal();
        is_fullscreen = false;
        //show hidden widgets
        ui->toolBar->show();
        ui->statusBar->show();
        ui->tabWidget->show();
        hidePlaylist();
        ui->hideButton->setEnabled(true);
        //show border
        playerLayout->setMargin(4);
    }
    else
    {
        showFullScreen();
        is_fullscreen = true;
        //hide other widgets
        ui->toolBar->hide();
        ui->statusBar->hide();
        ui->tabWidget->hide();
        playlist->hide();
        ui->hideButton->setEnabled(false);
        //hide border
        playerLayout->setMargin(0);
        //set auto-hide toolbar
        toolbar_visible = false;
        toolbar_pos_y = QApplication::desktop()->height() - ui->toolBar->height();
    }
}

//Auto hide / show toolbar when fullscreen
bool Player::eventFilter(QObject *, QEvent *e)
{
    if (!is_fullscreen)
        return false;

    else if (e->type() == QEvent::MouseMove)
    {
        QMouseEvent* me = static_cast<QMouseEvent*>(e);
        if (me->y() > toolbar_pos_y && !toolbar_visible) //mouse enters toolbar
        {
            ui->toolBar->show();
            toolbar_visible = true;
        }
        else if (toolbar_visible) //mouse leaves toolbar
        {
            ui->toolBar->hide();
            toolbar_visible = false;
        }
        return true;
    }
    return false;
}

//Show catching message
void Player::onCatching(bool b)
{
    if (b)
        ui->statusBar->showMessage(tr("Catching, Please wait..."));
    else
        ui->statusBar->clearMessage();
}

//Stop playing
void Player::onStopButton()
{
    no_play_next = true;
    mplayer->stop();
}

void Player::onStopped()
{
    setIconToPlay();
    if (no_play_next)
        no_play_next = false;
    else
        playlist->playNext();
}

void Player::onNeedPause(bool b)
{
    if (b && mplayer->state == VIDEO_PLAYING)
        mplayer->changeState();
    else if (!b && mplayer->state == VIDEO_PAUSING)
        mplayer->changeState();
}

void Player::setIconToPlay()
{
    ui->playButton->setIcon(skin[SKIN_PLAY]);
}

void Player::setIconToPause()
{
    ui->playButton->setIcon(skin[SKIN_PAUSE]);
}

void Player::onLengthChanged(int len)
{
    ui->progressBar->setMaximum(len / MPlayer::UPDATE_FREQUENCY);
    ui->stackedWidget->setCurrentIndex(0); //This func is called when a new video start to play
}

void Player::onPBarChanged(int pos)
{
    mplayer->setProgress(pos * MPlayer::UPDATE_FREQUENCY);
}

void Player::onProgressChanged(int pos)
{
    ui->progressBar->setValue(pos / MPlayer::UPDATE_FREQUENCY);
}

void Player::changeTabToPlayer()
{
    ui->stackedWidget->setCurrentIndex(0);
}

void Player::changeTabToWeb()
{
    ui->stackedWidget->setCurrentIndex(1);
}

void Player::setSkin(const QString& skin_name)
{
    QDir dir(QDir::currentPath() + "/skins/" + skin_name);

    skin[SKIN_PLAY] = QPixmap(dir.filePath("play.png"));
    skin[SKIN_STOP] = QPixmap(dir.filePath("stop.png"));
    skin[SKIN_PAUSE] = QPixmap(dir.filePath("pause.png"));

    ui->playButton->setIcon(skin[SKIN_PLAY]);
    ui->playButton->setIconSize(skin[SKIN_PLAY].size());
    ui->playButton->setMaximumSize(skin[SKIN_PLAY].size());

    ui->stopButton->setIcon(skin[SKIN_STOP]);
    ui->stopButton->setIconSize(skin[SKIN_STOP].size());
    ui->stopButton->setMaximumSize(skin[SKIN_STOP].size());

}
